'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var babelPluginMacros = require('babel-plugin-macros');
var helperModuleImports = require('@babel/helper-module-imports');
var babelUtils = require('@emotion/babel-utils');
var css = _interopDefault(require('@emotion/css'));

var transformCssCallExpression = function transformCssCallExpression(_ref) {
  var babel = _ref.babel,
      state = _ref.state,
      path = _ref.path;
  var t = babel.types;
  var sourceMap = '';

  if (t.isTaggedTemplateExpression(path)) {
    var expressions = babelUtils.getExpressionsFromTemplateLiteral(path.node.quasi, t);

    if (state.emotionSourceMap && path.node.quasi.loc !== undefined) {
      sourceMap = babelUtils.getSourceMap(path.node.quasi.loc.start, state);
    }

    path.replaceWith(t.callExpression(path.node.tag, expressions));
  }

  if (t.isCallExpression(path)) {
    var label = babelUtils.getLabelFromPath(path, t);
    var isPure = true;
    path.get('arguments').forEach(function (node) {
      if (!node.isPure()) {
        isPure = false;
      }

      if (t.isObjectExpression(node)) {
        node.replaceWith(babelUtils.simplifyObject(node.node, t));
      }
    });

    if (label) {
      babelUtils.appendStringToExpressions(path.node.arguments, "label:" + label + ";", t);
    }

    if (state.emotionSourceMap) {
      if (!sourceMap && path.node.loc !== undefined) {
        sourceMap = babelUtils.getSourceMap(path.node.loc.start, state);
      }

      babelUtils.appendStringToExpressions(path.node.arguments, sourceMap, t);
    }

    path.node.arguments = babelUtils.joinStringLiterals(path.node.arguments, t);

    if (path.node.arguments.length === 1 && t.isStringLiteral(path.node.arguments[0])) {
      var cssString = path.node.arguments[0].value;
      var res = css(cssString);
      path.replaceWith(t.objectExpression([t.objectProperty(t.identifier('name'), t.stringLiteral(res.name)), t.objectProperty(t.identifier('styles'), t.stringLiteral(res.styles))]));
    }

    if (isPure) {
      path.hoist();
    }
  }
};
var cssMacro = babelPluginMacros.createMacro(function (_ref2) {
  var references = _ref2.references,
      state = _ref2.state,
      babel = _ref2.babel;
  var t = babel.types;

  if (references.default.length) {
    references.default.reverse().forEach(function (reference) {
      if (!state.cssIdentifier) {
        state.cssIdentifier = helperModuleImports.addDefault(reference, '@emotion/css', {
          nameHint: 'css'
        });
      }

      reference.replaceWith(t.cloneDeep(state.cssIdentifier));
      transformCssCallExpression({
        babel: babel,
        state: state,
        path: reference.parentPath
      });
    });
  }
});

var styledMacro = babelPluginMacros.createMacro(function (_ref) {
  var references = _ref.references,
      state = _ref.state,
      babel = _ref.babel;
  var t = babel.types;

  if (references.default.length) {
    var styledIdentifier;
    references.default.forEach(function (reference) {
      if (!styledIdentifier) {
        styledIdentifier = helperModuleImports.addDefault(reference, '@emotion/styled-base', {
          nameHint: 'styled'
        });
      }

      if (t.isMemberExpression(reference.parent)) {
        reference.parentPath.replaceWith(t.callExpression(t.cloneDeep(styledIdentifier), [t.stringLiteral(reference.parent.property.name)]));
      } else {
        reference.replaceWith(t.cloneDeep(styledIdentifier));
      }

      var sourceMap = '';

      if (reference.parentPath && reference.parentPath.parentPath) {
        var styledCallPath = reference.parentPath.parentPath;

        if (t.isTaggedTemplateExpression(styledCallPath)) {
          var expressions = babelUtils.getExpressionsFromTemplateLiteral(styledCallPath.node.quasi, t);

          if (state.emotionSourceMap && styledCallPath.node.quasi.loc !== undefined) {
            sourceMap = babelUtils.getSourceMap(styledCallPath.node.quasi.loc.start, state);
          }

          styledCallPath.replaceWith(t.callExpression(styledCallPath.node.tag, expressions));
        }
      }

      if (t.isCallExpression(reference.parentPath)) {
        reference.parentPath.node.arguments[1] = t.objectExpression([t.objectProperty(t.identifier('target'), t.stringLiteral(babelUtils.getTargetClassName(state, t))), t.objectProperty(t.identifier('label'), t.stringLiteral(babelUtils.getLabelFromPath(reference.parentPath, t)))]);
      }

      if (t.isCallExpression(reference.parentPath.parentPath)) {
        if (state.emotionSourceMap) {
          if (!sourceMap && reference.parentPath.parentPath.node.loc !== undefined) {
            sourceMap = babelUtils.getSourceMap(reference.parentPath.parentPath.node.loc.start, state);
          }

          babelUtils.appendStringToExpressions(reference.parentPath.parentPath.node.arguments, sourceMap, t);
        }
      }
    });
  }
});

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

// most of this is from https://github.com/kentcdodds/babel-plugin-macros/blob/master/src/index.js
var createPlugin = function createPlugin(macros, customVisitor) {
  return function (babel) {
    return {
      name: '@emotion/babel-plugin-core',
      visitor: _extends({
        ImportDeclaration: function ImportDeclaration(path, state) {
          if (macros[path.node.source.value] === undefined) {
            return;
          }

          var imports = path.node.specifiers.map(function (s) {
            return {
              localName: s.local.name,
              importedName: s.type === 'ImportDefaultSpecifier' ? 'default' : s.imported.name
            };
          });
          var shouldExit = false;
          var hasReferences = false;
          var referencePathsByImportName = imports.reduce(function (byName, _ref) {
            var importedName = _ref.importedName,
                localName = _ref.localName;
            var binding = path.scope.getBinding(localName);

            if (!binding) {
              shouldExit = true;
              return byName;
            }

            byName[importedName] = binding.referencePaths;
            hasReferences = hasReferences || Boolean(byName[importedName].length);
            return byName;
          }, {});

          if (!hasReferences || shouldExit) {
            return;
          }
          /**
           * Other plugins that run before babel-plugin-macros might use path.replace, where a path is
           * put into its own replacement. Apparently babel does not update the scope after such
           * an operation. As a remedy, the whole scope is traversed again with an empty "Identifier"
           * visitor - this makes the problem go away.
           *
           * See: https://github.com/kentcdodds/import-all.macro/issues/7
           */


          state.file.scope.path.traverse({
            Identifier: function Identifier() {}
          });
          macros[path.node.source.value]({
            references: referencePathsByImportName,
            state: state,
            babel: babel,
            isBabelMacrosCall: true
          });
          path.remove();
        }
      }, customVisitor(babel))
    };
  };
};

var macros = {
  css: cssMacro,
  styled: styledMacro
};
var appendStringToExpressions = function appendStringToExpressions(expressions, string, t) {
  if (!string) {
    return expressions;
  }

  if (t.isStringLiteral(expressions[expressions.length - 1])) {
    expressions[expressions.length - 1].value += string;
  } else {
    expressions.push(t.stringLiteral(string));
  }

  return expressions;
};
var index = createPlugin({
  '@emotion/css': cssMacro,
  '@emotion/styled': styledMacro
}, function (babel) {
  var t = babel.types;
  return {
    Program: function Program(path, state) {
      if (state.opts.jsx === undefined) {
        for (var _iterator = path.node.body, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
          var _ref;

          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
          }

          var _node = _ref;

          if (t.isImportDeclaration(_node) && _node.source.value === '@emotion/jsx') {
            state.transformCssProp = true;
            break;
          }
        }
      } else {
        state.transformCssProp = state.opts.jsx;
      }

      if (state.opts.sourceMap) {
        state.emotionSourceMap = true;
      }
    },
    JSXAttribute: function JSXAttribute(path, state) {
      if (path.node.name.name !== 'css' && state.transformCssProp) {
        return;
      }

      if (t.isJSXExpressionContainer(path.node.value) && (t.isObjectExpression(path.node.value.expression) || t.isArrayExpression(path.node.value.expression))) {
        var expressionPath = path.get('value.expression');

        if (expressionPath.isPure()) {
          if (!state.cssIdentifier) {
            state.cssIdentifier = helperModuleImports.addDefault(path, '@emotion/css', {
              nameHint: 'css'
            });
          }

          expressionPath.replaceWith(t.callExpression(t.cloneDeep(state.cssIdentifier), [path.node.value.expression, state.emotionSourceMap && path.node.loc !== undefined && t.stringLiteral(babelUtils.getSourceMap(path.node.loc.start, state))].filter(Boolean)));
          transformCssCallExpression({
            babel: babel,
            state: state,
            path: expressionPath
          });
        }
      }
    }
  };
});

exports.macros = macros;
exports.appendStringToExpressions = appendStringToExpressions;
exports.default = index;
//# sourceMappingURL=index.cjs.js.map
